import hashlib
import os, time, sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
import general as g

def dir_exists(path):
  return os.path.isdir(path)

def file_exists(path):
  return os.path.isfile(path)

def path_exists(path):
  return os.path.exists(path)

def is_hidden_path(p):
  fnames = p.split('/')
  ret = False
  for f in fnames:
    if len(f) > 0 and f[0] == '.' and f != '.':
      ret = True
  return ret

# basic ignore function: no hidden files, links, locks, log files, item has to be a file, no root/hash
def default_ignore(full_path, data_root_path = None):
  if not path_exists(full_path):
    return True
  if is_hidden_path(full_path):
    return True
  if os.path.islink(full_path):
    return True

  # ignore lock files and log files
  x = full_path.split('/')[-1]
  if x in [
    'log.txt',
    'log.txt.hash',
  ]:
    return True
  f_type = x.split('.')[-1]
  f_type2 = None
  if len(x.split('.')) >= 2:
    f_type2 = x.split('.')[-2]
  if f_type in [
    'lock',
    'sock',
  ] or (
    f_type == 'hash' and f_type2 in ['lock', 'sock']
  ):
    return True

  # ignore root/hash
  rpath = os.path.relpath(full_path, data_root_path)
  if len(rpath.split('/')) > 0 and rpath.split('/')[0] == 'hash':
    return True
  return False

def data_to_hash_filename_mapper(fnom):
  return fnom + '.hash'

def hash_to_data_filename_mapper(fnom):
  tmp = fnom.split('.hash')
  if tmp[-1] == '':
    # remove any trailing '.hash' in the filename
    return '.hash'.join(tmp[:-1])
  else:
    return fnom

def compute_digest(plaintext):
  return hashlib.md5(plaintext.encode('utf-8')).hexdigest()

def compute_digest_from_file(fnom):
  return hashlib.md5(open(fnom, 'rb').read()).hexdigest()

# hash the file data_file_path and put into hash_root_path, maintaining the same folder structure
# so that data_root_path maps to hash_root_path
def chash_file(
  data_file_path,
  data_root_path,
  hash_root_path,
  ignore_funcs = [default_ignore],
  write_output = True
):
  rpath = os.path.relpath(data_file_path, data_root_path)
  hpath = data_to_hash_filename_mapper(os.path.join(hash_root_path, rpath))
  for func in ignore_funcs:
    if func(data_file_path, data_root_path):
      return
  hash_digest = compute_digest_from_file(data_file_path)
  if write_output:
    g.write_file(hpath, hash_digest)

# hash all the files in a given folder
def chash_all_files(
  data_root_path,
  hash_root_path,
  ignore_funcs = [default_ignore],
  write_output = True,
  mute = False
):
  progress_pcts = [ 0.01 ] + [ 0.1 * (i+1) for i in range(10) ]
  total_file_ct = sum([len(files) for dirpath, dirs, files in os.walk(data_root_path)])
  progress_cts = [ int(total_file_ct*pct) for pct in progress_pcts ]
  current_pointer = 0
  current_ct = 0
  sub_start_time = time.time()
  for dirpath, dirs, files in os.walk(data_root_path):
    rpath = os.path.relpath(dirpath, data_root_path)
    hpath = os.path.join(hash_root_path, rpath)
    if is_hidden_path(rpath):
      if not mute:
        print("skipping path %s" % rpath)
    else:
      if not os.path.isdir(hpath):
        os.makedirs(hpath)
      for f in files:
        current_ct += 1
        if(current_ct > progress_cts[current_pointer]):
          current_pct = progress_pcts[current_pointer]
          sys.stdout.write("\r%d files (%0.2f percent) hashed in %0.2fs. " % (
            current_ct, current_pct*100, (time.time() - sub_start_time)
          ) + "Estimated total time for completion: %s" % (
            "{:,}".format(int((time.time() - sub_start_time)/current_pct))
          ))
          sys.stdout.flush()
          current_pointer += 1

        fpath = os.path.join(dirpath, f)
        chash_file(fpath, data_root_path, hash_root_path, ignore_funcs)
  print("")

# paginate the hashes so there aren't a billion files
# def do_chash_paginated():


import os

f = open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'VERSION.txt'))
s = f.read()
f.close()

__version__ = s

import siweilib.general
import siweilib.general as g
import siweilib.filez
import siweilib.filez as f


# tests for chash

import sys, os
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
import filez.lib as lib
import general as g
import random, hashlib

filez_dir = os.path.dirname(os.path.abspath(__file__))
filez_test_dir = os.path.join(filez_dir, 'test')
filez_test_hash_dir = os.path.join(filez_dir, 'test', 'hash')

hash_test_data = {
  '0.txt': 'DCO7I5jdCfIj6PaH8gmiftjUcYyzPgKmkrVlgDXtnexPzfmD4doJyt9yI1HSCMD3VecxsCL6dxOlhNa4RHcnWy7QJB6YrM7YoVy4',
  '1.txt': '[VfrCT>numgS=LTl4M1bQ(Wh|<#t?<ZDD84kR};vmJuX1E}z(o@S^UIUjnM|Z7hjvGD?g(OcV-:|CP2G=+%7Zi8(L9K2XPPTgt[$',
  '2.bin': [176, 174, 96, 108, 53, 196, 169, 39, 160, 69, 97, 225, 102, 138, 40, 125, 108, 200, 102, 242, 245, 109, 133, 139, 144, 183, 62, 198, 104, 93, 117, 169, 160, 229, 41, 142, 196, 242, 192, 44, 253, 92, 91, 243, 84, 234, 46, 185, 243, 194, 217, 33, 249, 160, 0, 8, 255, 46, 28, 213, 160, 148, 212, 127, 43, 142, 51, 129, 129, 247, 90, 111, 39, 30, 215, 10, 141, 248, 56, 231, 204, 88, 111, 194, 71, 135, 239, 90, 183, 205, 97, 63, 104, 147, 68, 58, 175, 173, 154, 115],
  '3.bin': [random.randint(0, 255) for i in range(100)],
}

hash_expected_values = {
  '0.txt': '88fe9ea46eabe47af51ec6307aa4e350',
  '1.txt': '8a4af0b9a7fa9c1f17ccee4d1d8f5997',
  '2.bin': '85a6afe2cc55acb1d466dc8053a30f60',
  '3.bin': hashlib.md5(bytes(bytearray(hash_test_data['3.bin']))).hexdigest(),
}

def clean_up_test():
  os.system(f"rm -rf {filez_test_dir}")

def set_up_tests():
  clean_up_test()
  os.system(f"mkdir {filez_test_dir}")
  os.system(f"mkdir {filez_test_hash_dir}")

  for k, v in hash_test_data.items():
    if k.split('.')[-1] == 'txt':
      g.write_file(os.path.join(filez_test_dir, k), v)
    else:
      # binary file
      f = open(os.path.join(filez_test_dir, k), 'wb')
      f.write(bytearray(v))
      f.close()

def run_test():
  set_up_tests()
  os.system(f"python3.7 {os.path.join(filez_dir, 'chash.py')} -d {filez_test_dir} -o {filez_test_hash_dir}")
  # read in hashes and compare to original
  for k, v in hash_expected_values.items():
    hashed_value = g.read_file(os.path.join(filez_test_hash_dir, f"{k}.hash"))
    if hashed_value != v:
      return False
  return True

if __name__ == "__main__":
  if not run_test():
    print(g.error_message("Test filez.test failed!"))
  else:
    print(g.ok_message("Test filez.test passed"))

  clean_up_test()


import setuptools
import upgrade_version

version_str = upgrade_version.get_version()

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="siweilib",
    version=version_str,
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(exclude=("tests",)),
    python_requires='>=3.5',
    install_requires=[
        "termcolor",
        "pytz",
        "argparse",
        "cryptography",
    ]
)



# from . import lib
# from .. import general as g
# ^^ this works if importing but not if you're running the script... because of guido... orz
import sys, os

sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
import filez.lib as lib
import general as g

import argparse

def chash_main(hash_path, output_path = None, mute = False):
  if hash_path:
    # if output not specified, hash to $data_folder/hash
    # therefore, don't hash this location if it exists ^

    # check location exists
    if not lib.dir_exists(hash_path):
      print(g.error_message(f"Path {hash_path} does not exist"))
      exit()

    if not output_path:
      output_path = os.path.join(hash_path, "hash")
      if dir_exists(output_path):
        print(g.highlight_message(f"Destination folder {output_path} exists, (potentially) overwriting"))

    lib.chash_all_files(hash_path, output_path, write_output = True, mute = mute)
  else:
    print(g.error_message(f"Hash path not specified"))

  return None

if __name__ == "__main__":

  parser = argparse.ArgumentParser(description="""
    low level hash tool: creates hashes of folders for comparison purposes

    Examples:

      chash -d $path_to_data_folder -o $path_to_hash_folder
    """,
    formatter_class=argparse.RawTextHelpFormatter
  )

  parser.add_argument('-d', '--hash-path', action='store', help="data root location (absolute path)")
  parser.add_argument('-o', '--output-path', nargs='?', action='store', help="hash root location (absolute path)")
  parser.add_argument('-m', '--mute', action='store_true', help="do not ask for user inputs, just run")

  args = parser.parse_args()

  chash_main(
    hash_path = args.hash_path,
    output_path = args.output_path,
    mute = args.mute
  )